@@ -3,6 +3,7 @@
 
 //! CLI argument parsing structures and enums.
 
+use super::clap_error::EarlyArgs;
 use crate::{
     ExpectedError, Result,
     cargo_cli::{CargoCli, CargoOptions},
@@ -40,7 +41,7 @@ use nextest_runner::{
     },
     test_filter::{FilterBound, RunIgnored, TestFilterBuilder, TestFilterPatterns},
     test_output::CaptureStrategy,
-    user_config::elements::{DefaultUiConfig, UiConfig, UiShowProgress},
+    user_config::elements::{PagerSetting, PaginateSetting, UiConfig, UiShowProgress},
 };
 use std::{collections::BTreeSet, io::Cursor, sync::Arc, time::Duration};
 use tracing::{debug, warn};
@@ -277,10 +278,39 @@ pub(super) struct ListOpts {
     )]
     pub(super) list_type: ListType,
 
+    #[clap(flatten)]
+    pub(super) pager_opts: PagerOpts,
+
     #[clap(flatten)]
     pub(super) reuse_build: ReuseBuildOpts,
 }
 
+/// Pager options for list output.
+#[derive(Debug, Default, Args)]
+#[command(next_help_heading = "Pager options")]
+pub(super) struct PagerOpts {
+    /// Disable paging for this invocation.
+    #[arg(long)]
+    no_pager: bool,
+}
+
+impl PagerOpts {
+    /// Returns the effective pager and paginate settings, given the resolved UI
+    /// config.
+    ///
+    /// If `--no-pager` is specified, returns `PaginateSetting::Never`.
+    /// Otherwise, falls back to the resolved config values.
+    pub(super) fn resolve(&self, resolved_ui: &UiConfig) -> (PagerSetting, PaginateSetting) {
+        if self.no_pager {
+            // --no-pager disables paging entirely.
+            return (resolved_ui.pager.clone(), PaginateSetting::Never);
+        }
+
+        // Fall back to resolved config.
+        (resolved_ui.pager.clone(), resolved_ui.paginate)
+    }
+}
+
 #[derive(Debug, Args)]
 pub(super) struct RunOpts {
     #[clap(flatten)]
@@ -455,21 +485,16 @@ impl BenchReporterOpts {
     pub(super) fn to_builder(
         &self,
         should_colorize: bool,
-        user_ui_config: Option<&UiConfig>,
-        default_ui_config: &DefaultUiConfig,
+        resolved_ui: &UiConfig,
     ) -> ReporterBuilder {
         let mut builder = ReporterBuilder::default();
         builder.set_no_capture(true);
         builder.set_colorize(should_colorize);
-        // Determine show_progress with precedence: CLI/env > user config > default.
+        // Determine show_progress with precedence: CLI/env > resolved config.
         let ui_show_progress = self
             .show_progress
             .map(UiShowProgress::from)
-            .unwrap_or_else(|| {
-                user_ui_config
-                    .and_then(|c| c.show_progress)
-                    .unwrap_or(default_ui_config.show_progress)
-            });
+            .unwrap_or(resolved_ui.show_progress);
         if ui_show_progress == UiShowProgress::Only {
             // "only" implies --status-level=slow and --final-status-level=none.
             builder.set_status_level(StatusLevel::Slow);
@@ -537,6 +562,16 @@ impl MessageFormatOpts {
             Self::JsonPretty => OutputFormat::Serializable(SerializableFormat::JsonPretty),
         }
     }
+
+    /// Returns true if this format is human-readable (suitable for paging).
+    ///
+    /// Machine-readable formats (JSON) should not be paged.
+    pub(super) fn is_human_readable(self) -> bool {
+        match self {
+            Self::Auto | Self::Human | Self::Oneline => true,
+            Self::Json | Self::JsonPretty => false,
+        }
+    }
 }
 
 #[derive(Debug, Args)]
@@ -587,10 +622,11 @@ pub(super) struct TestBuildFilter {
 
     /// Test name filters and emulated test binary arguments.
     ///
-    /// Supported arguments:{n}
-    /// - --ignored:         Only run ignored tests{n}
-    /// - --include-ignored: Run both ignored and non-ignored tests{n}
-    /// - --skip PATTERN:    Skip tests that match the pattern{n}
+    /// Supported arguments:
+    ///
+    /// - --ignored:         Only run ignored tests
+    /// - --include-ignored: Run both ignored and non-ignored tests
+    /// - --skip PATTERN:    Skip tests that match the pattern
     /// - --exact:           Run tests that exactly match patterns after `--`
     #[arg(help_heading = None, value_name = "FILTERS_AND_ARGS", last = true)]
     filters: Vec<String>,
@@ -1186,8 +1222,7 @@ impl ReporterOpts {
         no_run: bool,
         no_capture: bool,
         should_colorize: bool,
-        user_ui_config: Option<&UiConfig>,
-        default_ui_config: &DefaultUiConfig,
+        resolved_ui: &UiConfig,
     ) -> ReporterBuilder {
         // Warn on conflicts between options. This is a warning and not an error
         // because these options can be specified via environment variables as
@@ -1221,7 +1256,7 @@ impl ReporterOpts {
             warn!("ignoring --message-format-version because --no-run is specified");
         }
 
-        // Determine show_progress with precedence: CLI/env > user config > default.
+        // Determine show_progress with precedence: CLI/env > resolved config.
         // Use UiShowProgress to preserve the "only" variant's special behavior.
         let ui_show_progress = match (self.show_progress, self.hide_progress_bar) {
             (Some(show_progress), true) => {
@@ -1230,29 +1265,17 @@ impl ReporterOpts {
             }
             (Some(show_progress), false) => show_progress.into(),
             (None, true) => UiShowProgress::None,
-            (None, false) => {
-                // Check user config before using default.
-                user_ui_config
-                    .and_then(|c| c.show_progress)
-                    .unwrap_or(default_ui_config.show_progress)
-            }
+            (None, false) => resolved_ui.show_progress,
         };
 
-        // Determine max_progress_running with precedence: CLI/env > user config > default.
+        // Determine max_progress_running with precedence: CLI/env > resolved config.
         let max_progress_running = self
             .max_progress_running
-            .or_else(|| user_ui_config.and_then(|c| c.max_progress_running))
-            .unwrap_or(default_ui_config.max_progress_running);
+            .unwrap_or(resolved_ui.max_progress_running);
 
-        // Note: CLI uses --no-output-indent (negative), user config uses
-        // output-indent (positive).
-        let no_output_indent = if self.no_output_indent {
-            true
-        } else if let Some(output_indent) = user_ui_config.and_then(|c| c.output_indent) {
-            !output_indent
-        } else {
-            !default_ui_config.output_indent
-        };
+        // Note: CLI uses --no-output-indent (negative), resolved config uses
+        // output_indent (positive).
+        let no_output_indent = self.no_output_indent || !resolved_ui.output_indent;
 
         debug!(
             ?ui_show_progress,
@@ -1413,6 +1436,10 @@ impl From<ShowProgressOpt> for UiShowProgress {
     max_term_width = 100,
 )]
 pub struct CargoNextestApp {
+    /// Early args (color, no_pager) flattened at root for early extraction.
+    #[clap(flatten)]
+    early_args: EarlyArgs,
+
     #[clap(subcommand)]
     subcommand: NextestSubcommand,
 }
@@ -1421,8 +1448,8 @@ impl CargoNextestApp {
     /// Initializes the output context.
     pub fn init_output(&self) -> OutputContext {
         match &self.subcommand {
-            NextestSubcommand::Nextest(args) => args.common.output.init(),
-            NextestSubcommand::Ntr(args) => args.common.output.init(),
+            NextestSubcommand::Nextest(args) => args.common.output.init(self.early_args),
+            NextestSubcommand::Ntr(args) => args.common.output.init(self.early_args),
             #[cfg(unix)]
             // Double-spawned processes should never use coloring.
             NextestSubcommand::DoubleSpawn(_) => OutputContext::color_never_init(),
@@ -1496,7 +1523,11 @@ impl AppOpts {
                     output_writer,
                 )?;
                 let app = super::execution::App::new(base, list_opts.build_filter)?;
-                app.exec_list(list_opts.message_format, list_opts.list_type, output_writer)?;
+                app.exec_list(
+                    list_opts.message_format,
+                    list_opts.list_type,
+                    &list_opts.pager_opts,
+                )?;
                 Ok(0)
             }
             Command::Run(run_opts) => {
@@ -1562,8 +1593,8 @@ impl AppOpts {
                 output,
                 output_writer,
             ),
-            Command::Self_ { command } => command.exec(self.common.output),
-            Command::Debug { command } => command.exec(self.common.output),
+            Command::Self_ { command } => command.exec(output),
+            Command::Debug { command } => command.exec(output),
         }
     }
 }
@@ -1672,6 +1703,11 @@ mod tests {
             "cargo nextest list --lib --bins",
             "cargo nextest run --ignore-rust-version --unit-graph",
             // ---
+            // Pager options
+            // ---
+            "cargo nextest list --no-pager",
+            "cargo nextest show-config test-groups --no-pager",
+            // ---
             // Reuse build options
             // ---
             "cargo nextest list --binaries-metadata=foo",
@@ -1713,6 +1749,17 @@ mod tests {
             "cargo nextest run --build-jobs -1",
             "cargo nextest run --build-jobs 1",
             // ---
+            // Self update options
+            // ---
+            "cargo nextest self update",
+            "cargo nextest self update --beta",
+            "cargo nextest self update --rc",
+            "cargo nextest self update --version 0.9.100",
+            "cargo nextest self update --version latest",
+            "cargo nextest self update --check",
+            "cargo nextest self update --beta --check",
+            "cargo nextest self update --rc --force",
+            // ---
             // Bench command
             // ---
             "cargo nextest bench",
@@ -1875,6 +1922,18 @@ mod tests {
             ("cargo nextest bench --stress-count 0", ValueValidation),
             // Invalid stress duration: 0
             ("cargo nextest bench --stress-duration 0m", ValueValidation),
+            // ---
+            // Self update option conflicts
+            // ---
+            ("cargo nextest self update --beta --rc", ArgumentConflict),
+            (
+                "cargo nextest self update --beta --version 0.9.100",
+                ArgumentConflict,
+            ),
+            (
+                "cargo nextest self update --rc --version 0.9.100",
+                ArgumentConflict,
+            ),
         ];
 
         // Unset all NEXTEST_ env vars because they can conflict with the try_parse_from below.
