@@ -6,9 +6,13 @@
 use super::{
     cli::{
         ArchiveBuildFilter, BenchReporterOpts, BenchRunnerOpts, ListType, MessageFormat,
-        MessageFormatOpts, NoTestsBehavior, ReporterOpts, TestBuildFilter, TestRunnerOpts,
+        MessageFormatOpts, NoTestsBehavior, PagerOpts, ReporterOpts, TestBuildFilter,
+        TestRunnerOpts,
+    },
+    helpers::{
+        acquire_graph_data, build_filtersets, detect_build_platforms, resolve_user_config,
+        runner_for_target,
     },
-    helpers::{acquire_graph_data, build_filtersets, detect_build_platforms, runner_for_target},
 };
 use crate::{
     ExpectedError, Result, ReuseBuildKind,
@@ -28,6 +32,7 @@ use nextest_runner::{
     helpers::plural,
     input::InputHandlerKind,
     list::{BinaryList, TestExecuteContext, TestList},
+    pager::PagedOutput,
     platform::BuildPlatforms,
     redact::Redactor,
     reporter::{
@@ -44,7 +49,7 @@ use nextest_runner::{
     target_runner::TargetRunner,
     test_filter::{BinaryFilter, TestFilterBuilder},
     test_output::CaptureStrategy,
-    user_config::{DefaultUserConfig, UserConfig},
+    user_config::elements::PaginateSetting,
     write_str::WriteStr,
 };
 use owo_colors::OwoColorize;
@@ -485,7 +490,7 @@ impl App {
         &self,
         message_format: MessageFormatOpts,
         list_type: ListType,
-        output_writer: &mut OutputWriter,
+        pager_opts: &PagerOpts,
     ) -> Result<()> {
         let pcx = ParseContext::new(self.base.graph());
 
@@ -499,18 +504,44 @@ impl App {
 
         let binary_list = self.base.build_binary_list("test")?;
 
+        // Resolve user config to get pager settings.
+        let resolved_user_config = resolve_user_config(&self.base.build_platforms.host.platform)?;
+        let (pager_setting, paginate) = pager_opts.resolve(&resolved_user_config.ui);
+
+        // Determine if we should page output.
+        // Don't page if:
+        //
+        // - paginate is Never
+        // - message_format is not human-readable
+        let should_page =
+            !matches!(paginate, PaginateSetting::Never) && message_format.is_human_readable();
+
+        // Create paged output if conditions are met.
+        let mut paged_output = if should_page {
+            PagedOutput::request_pager(
+                &pager_setting,
+                paginate,
+                &resolved_user_config.ui.streampager,
+            )
+        } else {
+            PagedOutput::terminal()
+        };
+
+        // (Grab the value of is_interactive before a mutable ref is passed in.)
+        let is_interactive = paged_output.is_interactive();
+        let should_colorize = self
+            .base
+            .output
+            .color
+            .should_colorize(supports_color::Stream::Stdout);
+
         match list_type {
             ListType::BinariesOnly => {
-                let mut writer = output_writer.stdout_writer();
                 binary_list.write(
-                    message_format.to_output_format(self.base.output.verbose, writer.is_terminal()),
-                    &mut writer,
-                    self.base
-                        .output
-                        .color
-                        .should_colorize(supports_color::Stream::Stdout),
+                    message_format.to_output_format(self.base.output.verbose, is_interactive),
+                    &mut paged_output,
+                    should_colorize,
                 )?;
-                writer.write_str_flush().map_err(WriteTestListError::Io)?;
             }
             ListType::Full => {
                 let double_spawn = self.base.load_double_spawn();
@@ -528,19 +559,20 @@ impl App {
                 let test_list =
                     self.build_test_list(&ctx, binary_list, test_filter_builder, &profile)?;
 
-                let mut writer = output_writer.stdout_writer();
                 test_list.write(
-                    message_format.to_output_format(self.base.output.verbose, writer.is_terminal()),
-                    &mut writer,
-                    self.base
-                        .output
-                        .color
-                        .should_colorize(supports_color::Stream::Stdout),
+                    message_format.to_output_format(self.base.output.verbose, is_interactive),
+                    &mut paged_output,
+                    should_colorize,
                 )?;
-                writer.write_str_flush().map_err(WriteTestListError::Io)?;
             }
         }
 
+        // Finalize the pager: close stdin/the pipe, wait for exit.
+        paged_output
+            .write_str_flush()
+            .map_err(WriteTestListError::Io)?;
+        paged_output.finalize();
+
         self.base
             .check_version_config_final(version_only_config.nextest_version())?;
         Ok(())
@@ -550,7 +582,7 @@ impl App {
         &self,
         show_default: bool,
         groups: Vec<nextest_runner::config::elements::TestGroup>,
-        output_writer: &mut OutputWriter,
+        pager_opts: &PagerOpts,
     ) -> Result<()> {
         let pcx = ParseContext::new(self.base.graph());
         let (_, config) = self.base.load_config(&pcx, &BTreeSet::new())?;
@@ -585,19 +617,33 @@ impl App {
 
         let test_list = self.build_test_list(&ctx, binary_list, test_filter_builder, &profile)?;
 
-        let mut writer = output_writer.stdout_writer();
+        // Resolve user config to get pager settings.
+        let resolved_user_config = resolve_user_config(&self.base.build_platforms.host.platform)?;
+        let (pager_setting, paginate) = pager_opts.resolve(&resolved_user_config.ui);
+
+        // Create paged output.
+        let mut paged_output = PagedOutput::request_pager(
+            &pager_setting,
+            paginate,
+            &resolved_user_config.ui.streampager,
+        );
 
         let show_test_groups = ShowTestGroups::new(&profile, &test_list, &settings);
         show_test_groups
             .write_human(
-                &mut writer,
+                &mut paged_output,
                 self.base
                     .output
                     .color
                     .should_colorize(supports_color::Stream::Stdout),
             )
             .map_err(WriteTestListError::Io)?;
-        writer.write_str_flush().map_err(WriteTestListError::Io)?;
+
+        // Finalize the pager: close stdin/the pipe, wait for exit.
+        paged_output
+            .write_str_flush()
+            .map_err(WriteTestListError::Io)?;
+        paged_output.finalize();
 
         Ok(())
     }
@@ -657,9 +703,8 @@ impl App {
             .color
             .should_colorize(supports_color::Stream::Stderr);
 
-        // Load user config for UI settings and get embedded defaults.
-        let user_config = UserConfig::from_default_location().map_err(Box::new)?;
-        let default_user_config = DefaultUserConfig::from_embedded();
+        // Load and resolve user config with platform-specific overrides.
+        let resolved_user_config = resolve_user_config(&self.base.build_platforms.host.platform)?;
 
         // Make the runner and reporter builders. Do them now so warnings are
         // emitted before we start doing the build.
@@ -668,8 +713,7 @@ impl App {
             runner_opts.no_run,
             no_capture || runner_opts.interceptor.is_active(),
             should_colorize,
-            user_config.as_ref().map(|c| &c.ui),
-            &default_user_config.ui,
+            &resolved_user_config.ui,
         );
         reporter_builder.set_verbose(self.base.output.verbose);
 
@@ -755,19 +799,12 @@ impl App {
             if reporter_opts.no_input_handler || runner_opts.interceptor.debugger.is_some() {
                 // Only debuggers disable the input handler -- tracers do not.
                 InputHandlerKind::Noop
+            } else if resolved_user_config.ui.input_handler {
+                // This means that the input handler determines whether it
+                // should be enabled.
+                InputHandlerKind::Standard
             } else {
-                // Check user config, falling back to embedded default.
-                let enabled = user_config
-                    .as_ref()
-                    .and_then(|c| c.ui.input_handler)
-                    .unwrap_or(default_user_config.ui.input_handler);
-                if enabled {
-                    // This means that the input handler determines whether it
-                    // should be enabled.
-                    InputHandlerKind::Standard
-                } else {
-                    InputHandlerKind::Noop
-                }
+                InputHandlerKind::Noop
             };
 
         // Make the runner.
@@ -832,18 +869,14 @@ impl App {
             .color
             .should_colorize(supports_color::Stream::Stderr);
 
-        // Load user config for UI settings and get embedded defaults.
-        let user_config = UserConfig::from_default_location().map_err(Box::new)?;
-        let default_user_config = DefaultUserConfig::from_embedded();
+        // Load and resolve user config with platform-specific overrides.
+        let resolved_user_config = resolve_user_config(&self.base.build_platforms.host.platform)?;
 
         // Make the runner and reporter builders. Do them now so warnings are
         // emitted before we start doing the build.
         let runner_builder = runner_opts.to_builder(cap_strat);
-        let mut reporter_builder = reporter_opts.to_builder(
-            should_colorize,
-            user_config.as_ref().map(|c| &c.ui),
-            &default_user_config.ui,
-        );
+        let mut reporter_builder =
+            reporter_opts.to_builder(should_colorize, &resolved_user_config.ui);
         reporter_builder.set_verbose(self.base.output.verbose);
 
         let filter_exprs =
@@ -928,17 +961,10 @@ impl App {
             if reporter_opts.no_input_handler || runner_opts.interceptor.debugger.is_some() {
                 // Only debuggers disable the input handler -- tracers do not.
                 InputHandlerKind::Noop
+            } else if resolved_user_config.ui.input_handler {
+                InputHandlerKind::Standard
             } else {
-                // Check user config, falling back to embedded default.
-                let enabled = user_config
-                    .as_ref()
-                    .and_then(|c| c.ui.input_handler)
-                    .unwrap_or(default_user_config.ui.input_handler);
-                if enabled {
-                    InputHandlerKind::Standard
-                } else {
-                    InputHandlerKind::Noop
-                }
+                InputHandlerKind::Noop
             };
 
         // Make the runner.
