@@ -23,11 +23,11 @@ fn test_failure_error() -> Result<(), String> {
 }
 
 fn nextest_attempt() -> usize {
-    static NEXTEST_ATTEMPT_ENV: &str = "__NEXTEST_ATTEMPT";
+    static NEXTEST_ATTEMPT_ENV: &str = "NEXTEST_ATTEMPT";
     match env::var(NEXTEST_ATTEMPT_ENV) {
         Ok(var) => var
             .parse()
-            .expect("__NEXTEST_ATTEMPT should be a positive integer"),
+            .expect("NEXTEST_ATTEMPT should be a positive integer"),
         Err(_) => 1,
     }
 }
@@ -144,6 +144,10 @@ macro_rules! assert_env {
     };
 }
 
+fn check_env(env: &str) -> String {
+    std::env::var(env).expect(&format!("{env} must be set"))
+}
+
 /// Assert that test environment variables are correctly set.
 #[test]
 fn test_cargo_env_vars() {
@@ -156,7 +160,7 @@ fn test_cargo_env_vars() {
         Ok("1"),
         "NEXTEST environment variable set to 1"
     );
-    std::env::var("NEXTEST_RUN_ID")
+    let run_id = std::env::var("NEXTEST_RUN_ID")
         .expect("NEXTEST_RUN_ID must be set")
         .parse::<uuid::Uuid>()
         .expect("NEXTEST_RUN_ID must be a UUID");
@@ -171,6 +175,40 @@ fn test_cargo_env_vars() {
         Ok("process-per-test"),
         "NEXTEST_EXECUTION_MODE set to process-per-test"
     );
+    assert_eq!(
+        std::env::var("NEXTEST_RUN_MODE").as_deref(),
+        Ok("test"),
+        "NEXTEST_RUN_MODE set to test"
+    );
+
+    // Assert NEXTEST_ATTEMPT is defined and is a positive integer
+    assert!(check_env("NEXTEST_ATTEMPT").parse::<usize>().is_ok());
+
+    let test_name = "test_cargo_env_vars";
+    assert_eq!(check_env("NEXTEST_TEST_NAME"), test_name);
+    let binary_id = "nextest-tests::basic";
+    assert_eq!(check_env("NEXTEST_BINARY_ID"), binary_id);
+
+    // The test might run with 1 or 3 total attempts
+    assert!(&["1", "3"].contains(&check_env("NEXTEST_TOTAL_ATTEMPTS").as_str()));
+
+    assert_eq!(check_env("NEXTEST_STRESS_CURRENT"), "none");
+    assert_eq!(check_env("NEXTEST_STRESS_TOTAL"), "none");
+
+    let attempt_id = check_env("NEXTEST_ATTEMPT_ID");
+
+    let (attempt_id_run_id, attempt_id) = attempt_id
+        .split_once(':')
+        .expect("NEXTEST_ATTEMPT_ID must contain ':'");
+    let attempt_id_run_id = attempt_id_run_id
+        .parse::<uuid::Uuid>()
+        .expect("NEXTEST_ATTEMPT_ID Run ID must be a UUID");
+    let (attempt_id_binary_id, attempt_id_test_name) = attempt_id
+        .split_once('$')
+        .expect("NEXTEST_ATTEMPT_ID must contain '$'");
+    assert_eq!(attempt_id_run_id, run_id);
+    assert_eq!(attempt_id_binary_id, binary_id);
+    assert_eq!(attempt_id_test_name, test_name);
 
     // https://doc.rust-lang.org/cargo/reference/environment-variables.html#environment-variables-cargo-sets-for-crates
 
@@ -304,7 +342,8 @@ fn test_slow_timeout() {
 #[test]
 #[ignore]
 fn test_slow_timeout_2() {
-    // There's a per-test override for the with-termination profile for this test: it is set to 1 second.
+    // There's a per-test override for the with-termination profile for this
+    // test: it is set to 1 second.
     std::thread::sleep(std::time::Duration::from_millis(1500));
 }
 
@@ -312,9 +351,9 @@ fn test_slow_timeout_2() {
 #[test]
 #[ignore]
 fn test_slow_timeout_subprocess() {
-    // Set a time greater than 5 seconds (that's the maximum amount the with_termination tests
-    // thinks tests should run for). Without job objects on Windows, the test wouldn't return until
-    // the sleep command exits.
+    // Set a time greater than 5 seconds (that's the maximum amount the
+    // with_termination tests thinks tests should run for). Without job objects
+    // on Windows, the test wouldn't return until the sleep command exits.
     let mut cmd = sleep_cmd(15);
     cmd.output().unwrap();
 }
@@ -326,8 +365,8 @@ fn test_flaky_slow_timeout_mod_3() {
     if nextest_attempt % 3 != 0 {
         panic!("Failed because attempt {} % 3 != 0", nextest_attempt)
     }
-    // The timeout for the with-timeout-success profile is set to 2 seconds.
-    std::thread::sleep(std::time::Duration::from_secs(4));
+    // The timeout for the with-timeout-retries-success profile is set to 1 second.
+    std::thread::sleep(std::time::Duration::from_millis(1500));
 }
 
 #[test]
