@@ -37,8 +37,11 @@ use target_spec::{Platform, summaries::TargetFeaturesSummary};
 mod fixtures;
 mod interceptor;
 mod large_alloc;
+#[cfg(unix)]
+mod sigttou;
 mod stuck_signal;
 mod temp_project;
+mod user_config;
 
 use crate::temp_project::{UdsStatus, create_uds};
 use camino_tempfile::Utf8TempDir;
