@@ -739,6 +739,20 @@ pub enum UserConfigError {
         #[source]
         error: FromPathBufError,
     },
+
+    /// Failed to compile a platform spec in an override.
+    #[error(
+        "for user config at {path}, failed to compile platform spec in [[overrides]] at index {index}"
+    )]
+    OverridePlatformSpec {
+        /// The path to the config file.
+        path: Utf8PathBuf,
+        /// The index of the override in the array.
+        index: usize,
+        /// The underlying target-spec error.
+        #[source]
+        error: target_spec::Error,
+    },
 }
 
 /// Error returned while parsing a [`MaxFail`](crate::config::elements::MaxFail) input.
@@ -1903,13 +1917,13 @@ pub enum InheritsError {
 #[cfg(feature = "self-update")]
 mod self_update_errors {
     use super::*;
+    use crate::update::PrereleaseKind;
     use mukti_metadata::ReleaseStatus;
     use semver::{Version, VersionReq};
 
     /// An error that occurs while performing a self-update.
     ///
     /// Returned by methods in the [`update`](crate::update) module.
-    #[cfg(feature = "self-update")]
     #[derive(Debug, Error)]
     #[non_exhaustive]
     pub enum UpdateError {
@@ -1949,6 +1963,17 @@ mod self_update_errors {
             req: VersionReq,
         },
 
+        /// No stable (non-prerelease) version was found.
+        #[error("no stable version found")]
+        NoStableVersion,
+
+        /// No version matching the requested prerelease kind was found.
+        #[error("no version found matching {} channel", kind.description())]
+        NoVersionForPrereleaseKind {
+            /// The kind of prerelease that was requested.
+            kind: PrereleaseKind,
+        },
+
         /// The specified mukti project was not found.
         #[error("project {not_found} not found in release metadata (known projects: {})", known.join(", "))]
         MuktiProjectNotFound {
@@ -2078,7 +2103,6 @@ mod self_update_errors {
         display_str
     }
 
-    #[cfg(feature = "self-update")]
     /// An error occurred while parsing an [`UpdateVersion`](crate::update::UpdateVersion).
     #[derive(Debug, Error)]
     pub enum UpdateVersionParseError {
