@@ -25,13 +25,13 @@
 use crate::{
     config::elements::{LeakTimeoutResult, SlowTimeoutResult},
     errors::{DisplayErrorChain, FormatVersionError, FormatVersionErrorInner, WriteEventError},
-    list::RustTestSuite,
+    list::{RustTestSuite, TestList},
     reporter::events::{ExecutionResult, StressIndex, TestEvent, TestEventKind},
     test_output::{ChildExecutionOutput, ChildOutput, ChildSingleOutput},
 };
 use bstr::ByteSlice;
 use iddqd::{IdOrdItem, IdOrdMap, id_ord_map, id_upcast};
-use nextest_metadata::{MismatchReason, RustBinaryId};
+use nextest_metadata::{MismatchReason, RustBinaryId, TestCaseName};
 use std::fmt::Write as _;
 
 /// To support pinning the version of the output, we just use this simple enum
@@ -141,6 +141,7 @@ fn fmt_err(err: std::fmt::Error) -> WriteEventError {
 pub struct LibtestReporter<'cfg> {
     _minor: FormatMinorVersion,
     _major: FormatMajorVersion,
+    test_list: Option<&'cfg TestList<'cfg>>,
     test_suites: IdOrdMap<LibtestSuite<'cfg>>,
     /// If true, we emit a `nextest` subobject with additional metadata in it
     /// that consumers can use for easier integration if they wish
@@ -168,6 +169,7 @@ impl<'cfg> LibtestReporter<'cfg> {
             return Ok(Self {
                 _minor: FormatMinorVersion::First,
                 _major: FormatMajorVersion::Unstable,
+                test_list: None,
                 test_suites: IdOrdMap::new(),
                 emit_nextest_obj,
             });
@@ -229,6 +231,7 @@ impl<'cfg> LibtestReporter<'cfg> {
         Ok(Self {
             _major: major,
             _minor: minor,
+            test_list: None,
             test_suites: IdOrdMap::new(),
             emit_nextest_obj,
         })
@@ -287,6 +290,10 @@ impl<'cfg> LibtestReporter<'cfg> {
                     test_instance,
                 )
             }
+            TestEventKind::RunStarted { test_list, .. } => {
+                self.test_list = Some(*test_list);
+                return Ok(());
+            }
             TestEventKind::StressSubRunFinished { .. } | TestEventKind::RunFinished { .. } => {
                 for test_suite in std::mem::take(&mut self.test_suites) {
                     self.finalize(test_suite)?;
@@ -297,7 +304,13 @@ impl<'cfg> LibtestReporter<'cfg> {
             _ => return Ok(()),
         };
 
-        let suite_info = test_instance.suite_info;
+        // Look up the suite info from the test list.
+        let test_list = self
+            .test_list
+            .expect("test_list should be set by RunStarted before any test events");
+        let suite_info = test_list
+            .get_suite(test_instance.binary_id)
+            .expect("suite should exist in test list");
         let crate_name = suite_info.package.name();
         let binary_name = &suite_info.binary_name;
 
@@ -351,7 +364,7 @@ impl<'cfg> LibtestReporter<'cfg> {
                     ignored,
                     filtered,
                     stress_index: *stress_index,
-                    meta: test_instance.suite_info,
+                    meta: suite_info,
                     total: std::time::Duration::new(0, 0),
                     ignore_block: None,
                     output_block: out,
@@ -390,7 +403,7 @@ impl<'cfg> LibtestReporter<'cfg> {
         if let Some(stress_index) = stress_index {
             write!(out, "@stress-{}", stress_index.current).map_err(fmt_err)?;
         }
-        write!(out, "${}", test_instance.name).map_err(fmt_err)?;
+        write!(out, "${}", test_instance.test_name).map_err(fmt_err)?;
         if let Some(retry_count) = retries {
             write!(out, "#{retry_count}\"").map_err(fmt_err)?;
         } else {
@@ -426,7 +439,7 @@ impl<'cfg> LibtestReporter<'cfg> {
                         strip_human_output_from_failed_test(
                             &last_status.output,
                             out,
-                            test_instance.name,
+                            test_instance.test_name,
                         )?;
                         out.extend_from_slice(b"\"");
                     }
@@ -457,7 +470,7 @@ impl<'cfg> LibtestReporter<'cfg> {
                     r#"{{"type":"{kind}","event":"{EVENT_IGNORED}","name":"{}::{}${}"}}"#,
                     suite_info.package.name(),
                     suite_info.binary_name,
-                    test_instance.name,
+                    test_instance.test_name,
                 )
                 .map_err(fmt_err)?;
             }
@@ -571,7 +584,7 @@ impl<'cfg> LibtestReporter<'cfg> {
 fn strip_human_output_from_failed_test(
     output: &ChildExecutionOutput,
     out: &mut bytes::BytesMut,
-    test_name: &str,
+    test_name: &TestCaseName,
 ) -> Result<(), WriteEventError> {
     match output {
         ChildExecutionOutput::Output {
@@ -638,7 +651,7 @@ fn strip_human_output_from_failed_test(
 fn strip_human_stdout_or_combined(
     output: &ChildSingleOutput,
     out: &mut bytes::BytesMut,
-    test_name: &str,
+    test_name: &TestCaseName,
 ) -> Result<(), WriteEventError> {
     if output.buf.contains_str("running 1 test\n") {
         // This is most likely the default test harness.
@@ -750,6 +763,7 @@ mod test {
     };
     use bytes::BytesMut;
     use color_eyre::eyre::eyre;
+    use nextest_metadata::TestCaseName;
     use std::{io, sync::Arc};
 
     /// Validates that the human output portion from a failed test is stripped
@@ -804,7 +818,7 @@ note: Some details are omitted, run with `RUST_BACKTRACE=full` for a verbose bac
                 errors: None,
             },
             &mut actual,
-            "index::test::download_url_crates_io",
+            &TestCaseName::new("index::test::download_url_crates_io"),
         )
         .unwrap();
 
@@ -835,7 +849,7 @@ note: Some details are omitted, run with `RUST_BACKTRACE=full` for a verbose bac
                 errors: None,
             },
             &mut actual,
-            "non-existent",
+            &TestCaseName::new("non-existent"),
         )
         .unwrap();
 
@@ -850,7 +864,12 @@ note: Some details are omitted, run with `RUST_BACKTRACE=full` for a verbose bac
         let output = ChildExecutionOutput::StartError(ChildStartError::Spawn(Arc::new(error)));
 
         let mut actual = bytes::BytesMut::new();
-        strip_human_output_from_failed_test(&output, &mut actual, "non-existent").unwrap();
+        strip_human_output_from_failed_test(
+            &output,
+            &mut actual,
+            &TestCaseName::new("non-existent"),
+        )
+        .unwrap();
 
         insta::assert_snapshot!(std::str::from_utf8(&actual).unwrap());
     }
@@ -868,7 +887,7 @@ note: Some details are omitted, run with `RUST_BACKTRACE=full` for a verbose bac
                 errors: None,
             },
             &mut actual,
-            "non-existent",
+            &TestCaseName::new("non-existent"),
         )
         .unwrap();
 
