@@ -9,10 +9,11 @@ use crate::{
     errors::TestFilterBuilderError,
     list::RustTestArtifact,
     partition::{Partitioner, PartitionerBuilder},
+    run_mode::NextestRunMode,
 };
 use aho_corasick::AhoCorasick;
 use nextest_filtering::{EvalContext, Filterset, TestQuery};
-use nextest_metadata::{FilterMatch, MismatchReason};
+use nextest_metadata::{FilterMatch, MismatchReason, RustTestKind, TestCaseName};
 use std::{collections::HashSet, fmt, mem};
 
 /// Whether to run ignored tests.
@@ -103,6 +104,7 @@ impl BinaryFilter {
 /// A builder for `TestFilter` instances.
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub struct TestFilterBuilder {
+    mode: NextestRunMode,
     run_ignored: RunIgnored,
     partitioner_builder: Option<PartitionerBuilder>,
     patterns: ResolvedFilterPatterns,
@@ -322,7 +324,8 @@ enum ResolvedFilterPatterns {
 }
 
 impl ResolvedFilterPatterns {
-    fn name_match(&self, test_name: &str) -> FilterNameMatch {
+    fn name_match(&self, test_name: &TestCaseName) -> FilterNameMatch {
+        let test_name = test_name.as_str();
         match self {
             Self::All => FilterNameMatch::MatchEmptyPatterns,
             Self::SkipOnly {
@@ -422,6 +425,7 @@ impl TestFilterBuilder {
     ///
     /// If an empty slice is passed, the test filter matches all possible test names.
     pub fn new(
+        mode: NextestRunMode,
         run_ignored: RunIgnored,
         partitioner_builder: Option<PartitionerBuilder>,
         patterns: TestFilterPatterns,
@@ -432,6 +436,7 @@ impl TestFilterBuilder {
         let binary_filter = BinaryFilter::new(exprs);
 
         Ok(Self {
+            mode,
             run_ignored,
             partitioner_builder,
             patterns,
@@ -454,9 +459,10 @@ impl TestFilterBuilder {
     }
 
     /// Creates a new `TestFilterBuilder` that matches the default set of tests.
-    pub fn default_set(run_ignored: RunIgnored) -> Self {
+    pub fn default_set(mode: NextestRunMode, run_ignored: RunIgnored) -> Self {
         let binary_filter = BinaryFilter::new(Vec::new());
         Self {
+            mode,
             run_ignored,
             partitioner_builder: None,
             patterns: ResolvedFilterPatterns::default(),
@@ -464,6 +470,11 @@ impl TestFilterBuilder {
         }
     }
 
+    /// Returns the nextest execution mode.
+    pub fn mode(&self) -> NextestRunMode {
+        self.mode
+    }
+
     /// Creates a new test filter scoped to a single binary.
     ///
     /// This test filter may be stateful.
@@ -585,58 +596,81 @@ impl TestFilter<'_> {
     pub fn filter_match(
         &mut self,
         test_binary: &RustTestArtifact<'_>,
-        test_name: &str,
+        test_name: &TestCaseName,
+        test_kind: &RustTestKind,
         ecx: &EvalContext<'_>,
         bound: FilterBound,
         ignored: bool,
     ) -> FilterMatch {
-        self.filter_ignored_mismatch(ignored)
-            .or_else(|| {
-                // ---
-                // NOTE
-                // ---
-                //
-                // Previously, if either expression OR string filters matched, we'd run the tests.
-                // The current (stable) implementation is that *both* the expression AND the string
-                // filters should match.
-                //
-                // This is because we try and skip running test binaries which don't match
-                // expression filters. So for example:
-                //
-                //     cargo nextest run -E 'binary(foo)' test_bar
-                //
-                // would not even get to the point of enumerating the tests not in binary(foo), thus
-                // not running any test_bars in the workspace. But, with the OR semantics:
-                //
-                //     cargo nextest run -E 'binary(foo) or test(test_foo)' test_bar
-                //
-                // would run all the test_bars in the repo. This is inconsistent, so nextest must
-                // use AND semantics.
-                use FilterNameMatch::*;
-                match (
-                    self.filter_name_match(test_name),
-                    self.filter_expression_match(test_binary, test_name, ecx, bound),
-                ) {
-                    // Tests must be accepted by both expressions and filters.
-                    (
-                        MatchEmptyPatterns | MatchWithPatterns,
-                        MatchEmptyPatterns | MatchWithPatterns,
-                    ) => None,
-                    // If rejected by at least one of the filtering strategies, the test is
-                    // rejected. Note we use the _name_ mismatch reason first. That's because
-                    // expression-based matches can also match against the default set. If a test
-                    // fails both name and expression matches, then the name reason is more directly
-                    // relevant.
-                    (Mismatch(reason), _) | (_, Mismatch(reason)) => {
-                        Some(FilterMatch::Mismatch { reason })
-                    }
+        if let Some(mismatch) = self.filter_benchmark_mismatch(test_kind) {
+            return mismatch;
+        }
+
+        if let Some(mismatch) = self.filter_ignored_mismatch(ignored) {
+            return mismatch;
+        }
+
+        {
+            // ---
+            // NOTE
+            // ---
+            //
+            // Previously, if either expression OR string filters matched, we'd run the tests.
+            // The current (stable) implementation is that *both* the expression AND the string
+            // filters should match.
+            //
+            // This is because we try and skip running test binaries which don't match
+            // expression filters. So for example:
+            //
+            //     cargo nextest run -E 'binary(foo)' test_bar
+            //
+            // would not even get to the point of enumerating the tests not in binary(foo), thus
+            // not running any test_bars in the workspace. But, with the OR semantics:
+            //
+            //     cargo nextest run -E 'binary(foo) or test(test_foo)' test_bar
+            //
+            // would run all the test_bars in the repo. This is inconsistent, so nextest must
+            // use AND semantics.
+            use FilterNameMatch::*;
+            match (
+                self.filter_name_match(test_name),
+                self.filter_expression_match(test_binary, test_name, ecx, bound),
+            ) {
+                // Tests must be accepted by both expressions and filters.
+                (
+                    MatchEmptyPatterns | MatchWithPatterns,
+                    MatchEmptyPatterns | MatchWithPatterns,
+                ) => {}
+                // If rejected by at least one of the filtering strategies, the test is
+                // rejected. Note we use the _name_ mismatch reason first. That's because
+                // expression-based matches can also match against the default set. If a test
+                // fails both name and expression matches, then the name reason is more directly
+                // relevant.
+                (Mismatch(reason), _) | (_, Mismatch(reason)) => {
+                    return FilterMatch::Mismatch { reason };
                 }
+            }
+        }
+
+        // Note that partition-based filtering MUST come after all other kinds
+        // of filtering, so that count-based bucketing applies after ignored,
+        // name and expression matching. This also means that mutable count
+        // state must be maintained by the partitioner.
+        if let Some(mismatch) = self.filter_partition_mismatch(test_name) {
+            return mismatch;
+        }
+
+        FilterMatch::Matches
+    }
+
+    fn filter_benchmark_mismatch(&self, test_kind: &RustTestKind) -> Option<FilterMatch> {
+        if self.builder.mode == NextestRunMode::Benchmark && test_kind != &RustTestKind::BENCH {
+            Some(FilterMatch::Mismatch {
+                reason: MismatchReason::NotBenchmark,
             })
-            // Note that partition-based filtering MUST come after all other kinds of filtering,
-            // so that count-based bucketing applies after ignored, name and expression matching.
-            // This also means that mutable count state must be maintained by the partitioner.
-            .or_else(|| self.filter_partition_mismatch(test_name))
-            .unwrap_or(FilterMatch::Matches)
+        } else {
+            None
+        }
     }
 
     fn filter_ignored_mismatch(&self, ignored: bool) -> Option<FilterMatch> {
@@ -660,14 +694,14 @@ impl TestFilter<'_> {
         None
     }
 
-    fn filter_name_match(&self, test_name: &str) -> FilterNameMatch {
+    fn filter_name_match(&self, test_name: &TestCaseName) -> FilterNameMatch {
         self.builder.patterns.name_match(test_name)
     }
 
     fn filter_expression_match(
         &self,
         test_binary: &RustTestArtifact<'_>,
-        test_name: &str,
+        test_name: &TestCaseName,
         ecx: &EvalContext<'_>,
         bound: FilterBound,
     ) -> FilterNameMatch {
@@ -699,9 +733,9 @@ impl TestFilter<'_> {
         }
     }
 
-    fn filter_partition_mismatch(&mut self, test_name: &str) -> Option<FilterMatch> {
+    fn filter_partition_mismatch(&mut self, test_name: &TestCaseName) -> Option<FilterMatch> {
         let partition_match = match &mut self.partitioner {
-            Some(partitioner) => partitioner.test_matches(test_name),
+            Some(partitioner) => partitioner.test_matches(test_name.as_str()),
             None => true,
         };
         if partition_match {
@@ -743,10 +777,17 @@ mod tests {
     #[proptest(cases = 50)]
     fn proptest_empty(#[strategy(vec(any::<String>(), 0..16))] test_names: Vec<String>) {
         let patterns = TestFilterPatterns::default();
-        let test_filter =
-            TestFilterBuilder::new(RunIgnored::Default, None, patterns, Vec::new()).unwrap();
+        let test_filter = TestFilterBuilder::new(
+            NextestRunMode::Test,
+            RunIgnored::Default,
+            None,
+            patterns,
+            Vec::new(),
+        )
+        .unwrap();
         let single_filter = test_filter.build();
         for test_name in test_names {
+            let test_name = TestCaseName::new(&test_name);
             prop_assert!(single_filter.filter_name_match(&test_name).is_match());
         }
     }
@@ -756,11 +797,18 @@ mod tests {
     fn proptest_exact(#[strategy(vec(any::<String>(), 0..16))] test_names: Vec<String>) {
         // Test with the default matcher.
         let patterns = TestFilterPatterns::new(test_names.clone());
-        let test_filter =
-            TestFilterBuilder::new(RunIgnored::Default, None, patterns, Vec::new()).unwrap();
+        let test_filter = TestFilterBuilder::new(
+            NextestRunMode::Test,
+            RunIgnored::Default,
+            None,
+            patterns,
+            Vec::new(),
+        )
+        .unwrap();
         let single_filter = test_filter.build();
         for test_name in &test_names {
-            prop_assert!(single_filter.filter_name_match(test_name).is_match());
+            let test_name = TestCaseName::new(test_name);
+            prop_assert!(single_filter.filter_name_match(&test_name).is_match());
         }
 
         // Test with the exact matcher.
@@ -768,11 +816,18 @@ mod tests {
         for test_name in &test_names {
             patterns.add_exact_pattern(test_name.clone());
         }
-        let test_filter =
-            TestFilterBuilder::new(RunIgnored::Default, None, patterns, Vec::new()).unwrap();
+        let test_filter = TestFilterBuilder::new(
+            NextestRunMode::Test,
+            RunIgnored::Default,
+            None,
+            patterns,
+            Vec::new(),
+        )
+        .unwrap();
         let single_filter = test_filter.build();
         for test_name in &test_names {
-            prop_assert!(single_filter.filter_name_match(test_name).is_match());
+            let test_name = TestCaseName::new(test_name);
+            prop_assert!(single_filter.filter_name_match(&test_name).is_match());
         }
     }
 
@@ -788,10 +843,17 @@ mod tests {
             patterns.add_substring_pattern(substring);
         }
 
-        let test_filter =
-            TestFilterBuilder::new(RunIgnored::Default, None, patterns, Vec::new()).unwrap();
+        let test_filter = TestFilterBuilder::new(
+            NextestRunMode::Test,
+            RunIgnored::Default,
+            None,
+            patterns,
+            Vec::new(),
+        )
+        .unwrap();
         let single_filter = test_filter.build();
         for test_name in test_names {
+            let test_name = TestCaseName::new(&test_name);
             prop_assert!(single_filter.filter_name_match(&test_name).is_match());
         }
     }
@@ -802,12 +864,23 @@ mod tests {
         prop_assume!(!substring.is_empty() && !prefix.is_empty() && !suffix.is_empty());
         let pattern = prefix + &substring + &suffix;
         let patterns = TestFilterPatterns::new(vec![pattern]);
-        let test_filter =
-            TestFilterBuilder::new(RunIgnored::Default, None, patterns, Vec::new()).unwrap();
+        let test_filter = TestFilterBuilder::new(
+            NextestRunMode::Test,
+            RunIgnored::Default,
+            None,
+            patterns,
+            Vec::new(),
+        )
+        .unwrap();
         let single_filter = test_filter.build();
+        let substring = TestCaseName::new(&substring);
         prop_assert!(!single_filter.filter_name_match(&substring).is_match());
     }
 
+    fn test_name(s: &str) -> TestCaseName {
+        TestCaseName::new(s)
+    }
+
     #[test]
     fn pattern_examples() {
         let mut patterns = TestFilterPatterns::new(vec!["foo".to_string()]);
@@ -820,57 +893,57 @@ mod tests {
 
         // Test substring matches.
         assert_eq!(
-            resolved.name_match("foo"),
+            resolved.name_match(&test_name("foo")),
             FilterNameMatch::MatchWithPatterns,
         );
         assert_eq!(
-            resolved.name_match("1foo2"),
+            resolved.name_match(&test_name("1foo2")),
             FilterNameMatch::MatchWithPatterns,
         );
         assert_eq!(
-            resolved.name_match("bar"),
+            resolved.name_match(&test_name("bar")),
             FilterNameMatch::MatchWithPatterns,
         );
         assert_eq!(
-            resolved.name_match("x_bar_y"),
+            resolved.name_match(&test_name("x_bar_y")),
             FilterNameMatch::MatchWithPatterns,
         );
 
         // Test exact matches.
         assert_eq!(
-            resolved.name_match("baz"),
+            resolved.name_match(&test_name("baz")),
             FilterNameMatch::MatchWithPatterns,
         );
         assert_eq!(
-            resolved.name_match("abazb"),
+            resolved.name_match(&test_name("abazb")),
             FilterNameMatch::Mismatch(MismatchReason::String),
         );
 
         // Both substring and exact matches.
         assert_eq!(
-            resolved.name_match("bazfoo"),
+            resolved.name_match(&test_name("bazfoo")),
             FilterNameMatch::MatchWithPatterns,
         );
 
         // Skip patterns.
         assert_eq!(
-            resolved.name_match("quux"),
+            resolved.name_match(&test_name("quux")),
             FilterNameMatch::Mismatch(MismatchReason::String),
         );
         assert_eq!(
-            resolved.name_match("1quux2"),
+            resolved.name_match(&test_name("1quux2")),
             FilterNameMatch::Mismatch(MismatchReason::String),
         );
 
         // Skip and substring patterns.
         assert_eq!(
-            resolved.name_match("quuxbar"),
+            resolved.name_match(&test_name("quuxbar")),
             FilterNameMatch::Mismatch(MismatchReason::String),
         );
 
         // Skip-exact patterns.
         assert_eq!(
-            resolved.name_match("quuz"),
+            resolved.name_match(&test_name("quuz")),
             FilterNameMatch::Mismatch(MismatchReason::String),
         );
 
@@ -878,7 +951,7 @@ mod tests {
         patterns.add_skip_pattern("baz".to_string());
         let resolved = patterns.resolve().unwrap();
         assert_eq!(
-            resolved.name_match("quuxbaz"),
+            resolved.name_match(&test_name("quuxbaz")),
             FilterNameMatch::Mismatch(MismatchReason::String),
         );
     }
@@ -894,35 +967,35 @@ mod tests {
 
         // Test substring matches.
         assert_eq!(
-            resolved.name_match("foo"),
+            resolved.name_match(&test_name("foo")),
             FilterNameMatch::Mismatch(MismatchReason::String),
         );
         assert_eq!(
-            resolved.name_match("1foo2"),
+            resolved.name_match(&test_name("1foo2")),
             FilterNameMatch::Mismatch(MismatchReason::String),
         );
         assert_eq!(
-            resolved.name_match("bar"),
+            resolved.name_match(&test_name("bar")),
             FilterNameMatch::Mismatch(MismatchReason::String),
         );
         assert_eq!(
-            resolved.name_match("x_bar_y"),
+            resolved.name_match(&test_name("x_bar_y")),
             FilterNameMatch::Mismatch(MismatchReason::String),
         );
 
         // Test exact matches.
         assert_eq!(
-            resolved.name_match("baz"),
+            resolved.name_match(&test_name("baz")),
             FilterNameMatch::Mismatch(MismatchReason::String),
         );
         assert_eq!(
-            resolved.name_match("abazb"),
+            resolved.name_match(&test_name("abazb")),
             FilterNameMatch::MatchWithPatterns,
         );
 
         // Anything that doesn't match the skip filter should match.
         assert_eq!(
-            resolved.name_match("quux"),
+            resolved.name_match(&test_name("quux")),
             FilterNameMatch::MatchWithPatterns,
         );
     }
@@ -935,27 +1008,27 @@ mod tests {
 
         // Anything matches.
         assert_eq!(
-            resolved.name_match("foo"),
+            resolved.name_match(&test_name("foo")),
             FilterNameMatch::MatchEmptyPatterns,
         );
         assert_eq!(
-            resolved.name_match("1foo2"),
+            resolved.name_match(&test_name("1foo2")),
             FilterNameMatch::MatchEmptyPatterns,
         );
         assert_eq!(
-            resolved.name_match("bar"),
+            resolved.name_match(&test_name("bar")),
             FilterNameMatch::MatchEmptyPatterns,
         );
         assert_eq!(
-            resolved.name_match("x_bar_y"),
+            resolved.name_match(&test_name("x_bar_y")),
             FilterNameMatch::MatchEmptyPatterns,
         );
         assert_eq!(
-            resolved.name_match("baz"),
+            resolved.name_match(&test_name("baz")),
             FilterNameMatch::MatchEmptyPatterns,
         );
         assert_eq!(
-            resolved.name_match("abazb"),
+            resolved.name_match(&test_name("abazb")),
             FilterNameMatch::MatchEmptyPatterns,
         );
     }
